""" This module implements specific error handler for Lobster runs. """

import os

from pymatgen.io.lobster import Lobsterout

from custodian.custodian import Validator

__author__ = "Janine George, Guido Petretto"
__copyright__ = "Copyright 2020, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Janine George"
__email__ = "janine.george@uclouvain.be"
__date__ = "April 27, 2020"


class EnoughBandsValidator(Validator):
    """
    validates if enough bands for COHP calculation are available
    """

    def __init__(self, output_filename: str = "lobsterout"):
        """

        Args:
            output_filename: filename of output file, usually lobsterout
        """
        self.output_filename = output_filename

    def check(self) -> bool:
        """
        checks if the VASP calculation had enough bands
        Returns:
            (bool) if True, too few bands have been applied
        """
        # checks if correct number of bands is available
        try:
            with open(self.output_filename) as f:
                data = f.read()
            return "You are employing too few bands in your PAW calculation." in data
        except OSError:
            return False


class LobsterFilesValidator(Validator):
    """
    Check for existence of some of the files that lobster
        normally create upon running.
    Check if lobster terminated normally by looking for finished
    """

    def __init__(self):
        """
        Dummy init
        """

    def check(self) -> bool:
        """
        Check for errors.
        """
        for vfile in ["lobsterout"]:
            if not os.path.exists(vfile):
                return True
        with open("lobsterout") as f:
            data = f.read()
        return "finished" not in data


class ChargeSpillingValidator(Validator):
    """
    Check if spilling is below certain threshold!
    """

    def __init__(self, output_filename: str = "lobsterout", charge_spilling_limit: float = 0.05):
        """

        Args:
            output_filename: filename of the output file of lobter, usually lobsterout
            charge_spilling_limit: limit of the charge spilling that will be considered okay
        """

        self.output_filename = output_filename
        self.charge_spilling_limit = charge_spilling_limit

    def check(self) -> bool:
        """open lobsterout and find charge spilling"""

        if os.path.exists(self.output_filename):
            lobsterout = Lobsterout(self.output_filename)
            if lobsterout.charge_spilling[0] > self.charge_spilling_limit:
                return True
            if len(lobsterout.charge_spilling) > 1:
                if lobsterout.charge_spilling[1] > self.charge_spilling_limit:
                    return True
            return False
        return False



"""
Implements various interpreters and modders for FEFF calculations.
"""

import os

from pymatgen.io.feff.sets import FEFFDictSet

from custodian.ansible.actions import DictActions, FileActions
from custodian.ansible.interpreter import Modder


class FeffModder(Modder):
    """
    A Modder for FeffInput sets
    """

    def __init__(self, actions=None, strict=True, feffinp=None):
        """
        Args:
            actions ([Action]): A sequence of supported actions. See
            actions ([Action]): A sequence of supported actions. See
                :mod:`custodian.ansible.actions`. Default is None,
                which means DictActions and FileActions are supported.
            strict (bool): Indicating whether to use strict mode. In non-strict
                mode, unsupported actions are simply ignored without any
                errors raised. In strict mode, if an unsupported action is
                supplied, a ValueError is raised. Defaults to True.
            feffinp (FEFFInput): A FeffInput object from the current directory.
                Initialized automatically if not passed (but passing it will
                avoid having to reparse the directory).
        """
        self.feffinp = feffinp or FEFFDictSet.from_directory(".")
        self.feffinp = self.feffinp.all_input()
        actions = actions or [FileActions, DictActions]
        super().__init__(actions, strict)

    def apply_actions(self, actions):
        """
        Applies a list of actions to the FEFF Input Set and rewrites modified
        files.

        Args:
            actions [dict]: A list of actions of the form {'file': filename,
                'action': moddermodification} or {'dict': feffinput_key,
                'action': moddermodification}
        """
        modified = []
        for a in actions:
            if "dict" in a:
                k = a["dict"]
                modified.append(k)
                self.feffinp[k] = self.modify_object(a["action"], self.feffinp[k])
            elif "file" in a:
                self.modify(a["action"], a["file"])
            else:
                raise ValueError(f"Unrecognized format: {a}")
        if modified:
            feff = self.feffinp
            feff_input = "\n\n".join(str(feff[k]) for k in ["HEADER", "PARAMETERS", "POTENTIALS", "ATOMS"] if k in feff)
            for k, v in feff.items():
                with open(os.path.join(".", k), "w") as f:
                    f.write(str(v))

            with open(os.path.join(".", "feff.inp"), "w") as f:
                f.write(feff_input)



"""
Custodian is a simple, robust and flexible just-in-time (JIT) job management
framework written in Python.
"""

__author__ = "Shyue Ping Ong, William Davidson Richards, Stephen Dacek, Xiaohui Qu, Matthew Horton, Samuel M. Blau"
__version__ = "2023.5.7"

from .custodian import Custodian



"""
Utility function and classes.
"""

import logging
import os
import tarfile
from glob import glob


def backup(filenames, prefix="error"):
    """
    Backup files to a tar.gz file. Used, for example, in backing up the
    files of an errored run before performing corrections.

    Args:
        filenames ([str]): List of files to backup. Supports wildcards, e.g.,
            *.*.
        prefix (str): prefix to the files. Defaults to error, which means a
            series of error.1.tar.gz, error.2.tar.gz, ... will be generated.
    """
    num = max([0] + [int(f.split(".")[1]) for f in glob(f"{prefix}.*.tar.gz")])
    filename = f"{prefix}.{num + 1}.tar.gz"
    logging.info(f"Backing up run to {filename}.")
    with tarfile.open(filename, "w:gz") as tar:
        for fname in filenames:
            for f in glob(fname):
                tar.add(f)


def get_execution_host_info():
    """
    Tries to return a tuple describing the execution host.
    Doesn't work for all queueing systems

    Returns:
        (HOSTNAME, CLUSTER_NAME)
    """
    host = os.environ.get("HOSTNAME", None)
    cluster = os.environ.get("SGE_O_HOST", None)
    if host is None:
        try:
            import socket

            host = host or socket.gethostname()
        except Exception:
            pass
    return host or "unknown", cluster or "unknown"
